#!/usr/bin/env python
#
# Browser bookmarks menu as a GNOME panel applet.
#
# (C) 2004 Nigel Tao.
# Licensed under the GNU GPL.
#
# TO DO:
# + global keyboard binding
# + persistent history
# + customizable engines

import gnome
import gnome.applet
import gnome.ui
import gobject
import gtk
import os
import sys



engines = [
	("f", "Search for Files",      "gnome-search-tool --path=$HOME --named=\"%s\" --start"),
	("d", "Dictionary",            "gnome-dictionary \"%s\""),
	(None, None, None),
	("t", "AltaVista",             "http://www.altavista.com/web/results?q=%s"),
	("o", "Amazon (A9)",           "http://www.a9.com/%s"),
	("b", "CDDB",                  "http://www.gracenote.com/music/search.html?f=artist&f=disc&q=%s"),
	("r", "Creative Commons",      "http://search.creativecommons.org/index.jsp?q=%s"),
	("e", "Ebay",                  "http://search.ebay.com/%s"),
	("g", "Google",                "http://www.google.com/search?q=%s"),
	("i", "IMDB",                  "http://www.imdb.com/find?q=%s"),
	("m", "MSN Search",            "http://search.msn.com/results.aspx?q=%s"),
	("p", "Open Directory (DMOZ)", "http://search.dmoz.org/cgi-bin/search?search=%s"),
	("q", "Stock Quotes (Yahoo)",  "http://finance.yahoo.com/q?s=%s"),
	("w", "Wikipedia",             "http://en.wikipedia.org/wiki/%s"),
	("y", "Yahoo",                 "http://search.yahoo.com/search?p=%s"),
	]
default_engine = "Google"


# Over-ride STOCK_FIND so that our find button has no text.
# See the PyGTK FAQ 9.5.
gtk.stock_add([(gtk.STOCK_FIND, None, 0, 0, None)])

gray = gtk.gdk.color_parse("gray")



class DeskbarApplet:
	def __init__(self):
		self.button = None
		self.entry = None
		self.list_store = None
		self.show_gray_text = False

		for engine in engines:
			(shortcut, name, cmd) = engine
			if name == default_engine:
				self.selected_engine = engine
				break


	def do_query(self, entry, query):
		if len(query) == 0:
			return
		
		self.list_store.append([query])
		query = query.replace("\"", "\\\"")
		cmd = self.selected_engine[2].replace("%s", query)
		if cmd.startswith("http:"):
			cmd = "gnome-open \"" + cmd + "\""
		os.system(cmd + " &")


	def on_applet_destroy(self, widget, arg):
		del widget


	def on_about(self, component, verb, applet):
		icon = gtk.Image()
		icon.set_from_file("/usr/share/pixmaps/gnome-mini-commander.png")

		fullname = "Deskbar Applet"
		version = "0.1"
		copyright = "Copyright (C) 2004 Nigel Tao"
		description = "A search bar."
		authors = ["Nigel Tao <nigel.tao@myrealbox.com>"]

		about = gnome.ui.About(fullname, version, copyright, description, authors, None, None, icon.get_pixbuf())
		about.set_icon(icon.get_pixbuf())
		about.show()


	def on_entry_activate(self, widget):
		if not self.show_gray_text:
			t = widget.get_text()
			widget.set_text("")
			self.do_query(widget, t)
		
		return gtk.FALSE


	def on_entry_focus_in(self, widget, event):
		if self.show_gray_text:
			self.show_gray_text = False
			widget.set_text("")
			widget.modify_text(gtk.STATE_NORMAL, None)
		
		widget.select_region(0, -1)
		return gtk.FALSE
	
	
	def on_entry_focus_out(self, widget, event):
		t = widget.get_text().strip()
		if len(t) == 0 or t == self.selected_engine[1]:
			self.set_show_gray_text()
		
		return gtk.FALSE


	def set_show_gray_text(self):
		self.show_gray_text = True
		self.entry.set_text(self.selected_engine[1])
		self.entry.select_region(0, 0)
		self.entry.modify_text(gtk.STATE_NORMAL, gray)


	def on_button_press(self, widget, event):
		# allow Middle- and Right-Mouse-Button to go through to the applet window
		if event.button != 1:
			widget.emit_stop_by_name("button-press-event")
		
		return gtk.FALSE


	def on_entry_key_press(self, entry, event):
		# bind Escape to clear the GtkEntry
		if event.keyval == gtk.keysyms.Escape:
			entry.set_text("")

		if event.get_state() & gtk.gdk.CONTROL_MASK:
			for engine in engines:
				(shortcut, name, cmd) = engine
				if shortcut != None and event.keyval == ord(shortcut):
					self.set_engine(engine)
					return gtk.TRUE
		
		return gtk.FALSE


	def build_menu(self):
		accel_group = gtk.AccelGroup()
		menu = gtk.Menu()
		group = None
		for engine in engines:
			(shortcut, name, cmd) = engine
			if cmd == None:
				menu_item = gtk.SeparatorMenuItem()
			else:
				menu_item = gtk.RadioMenuItem(group, name)
				menu_item.connect("activate", self.on_engine_menu_item_activate, engine)
				menu_item.add_accelerator("activate", accel_group, ord(shortcut), \
						gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
				if group == None:
					group = menu_item
				if name == default_engine:
					menu_item.set_active(True)
			menu.append(menu_item)
		menu.show_all()
		return menu


	def on_engine_menu_item_activate(self, widget, engine):
		if widget.get_active():
			self.set_engine(engine)


	def set_engine(self, engine):
		if self.selected_engine != engine:
			self.selected_engine = engine
			self.show_gray_text = False
			self.entry.set_text(self.selected_engine[1])
			self.entry.modify_text(gtk.STATE_NORMAL, None)
			self.entry.select_region(0, -1)
			self.entry.grab_focus()


	def on_applet_button_press(self, widget, event):
		# Left-Mouse-Button should focus the GtkEntry widget (for Fitt's Law
		# - so that a click on applet border on edge of screen activates the
		# most important widget).
		if event.button == 1:
			self.entry.grab_focus()
			return gtk.TRUE
		
		return gtk.FALSE


	def on_applet_size_allocate(self, widget, rect):
		self.entry.set_size_request(self.entry.size_request()[0], rect.height)
		return gtk.FALSE


def applet_factory(applet, iid):
	da = DeskbarApplet()
	
	box = gtk.HBox()
	box.set_border_width(0)
	
	da.entry = gtk.Entry()
	da.entry.connect("activate", da.on_entry_activate)
	da.entry.connect("focus-in-event", da.on_entry_focus_in)
	da.entry.connect("focus-out-event", da.on_entry_focus_out)
	da.entry.connect("key-press-event", da.on_entry_key_press)
	box.add(da.entry)
	
	da.set_show_gray_text()
	
	completion = gtk.EntryCompletion()
	da.entry.set_completion(completion)
	da.list_store = gtk.ListStore(gobject.TYPE_STRING)
	completion.set_model(da.list_store)
	completion.set_text_column(0)

	button = gtk.Button(stock=gtk.STOCK_FIND)
	button.connect_object("clicked", da.on_entry_activate, da.entry)
	button.connect("button-press-event", da.on_button_press)
	box.add(button)
	
	# TODO - replace this all by a MenuToolButton (new in PyGTK 2.6)
 	gtk.rc_parse_string('''
		style "deskbar-applet-menubar-style"
		{
			GtkMenuBar::shadow-type = none
			GtkMenuBar::internal-padding = 0
		}
		class "GtkMenuBar" style "deskbar-applet-menubar-style"''')
	menu_item = gtk.ImageMenuItem("")
	image = gtk.Image()
	image.set_from_file("/usr/share/pixmaps/mini-commander/history-mini.xpm")
	menu_item.set_image(image)
	menu_item.set_submenu(da.build_menu())
	menubar = gtk.MenuBar()
	menubar.append(menu_item)
	menubar.connect("button-press-event", da.on_button_press)
	# end TODO
	box.add(menubar)
	
	esr = da.entry.size_request()
	bsr = button.size_request()
	request_y = max(esr[1], bsr[1])
	da.entry.set_size_request(esr[0], request_y)
	button.set_size_request(bsr[0], request_y)
	
	applet.set_flags(gtk.CAN_FOCUS)
	applet.add(box)
	applet.connect("destroy", da.on_applet_destroy, None)
	applet.set_applet_flags(gnome.applet.EXPAND_MINOR)
	applet.connect("button-press-event", da.on_applet_button_press)
	applet.connect("size-allocate", da.on_applet_size_allocate)
	applet.show_all()

	# funky right-click menu
	menuXml = """
	<popup name="button3">
	        <menuitem
	        	name="Deskbar Applet About Item"
	        	verb="About"
	        	_label="_About"
	        	pixtype="stock"
	        	pixname="gnome-stock-about"/>
	</popup>
	"""
	applet.setup_menu(menuXml, [("About", da.on_about)], applet)
	
	return gtk.TRUE


if __name__ == "__main__":
	if len(sys.argv) == 2:
		if sys.argv[1] == "-print":
			print "No problems so far."

		elif sys.argv[1] == "-window":
			main_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
			main_window.set_title("Deskbar Applet")
			main_window.connect("destroy", gtk.main_quit)
			
			applet = gnome.applet.Applet()
			applet_factory(applet, None)
			applet.reparent(main_window)
			
			main_window.show_all()
			gtk.main()

	else:
		gnome.applet.bonobo_factory( \
			"OAFIID:Deskbar_Applet_Factory", \
			gnome.applet.Applet.__gtype__, \
			"deskbar_applet", \
			"0", \
			applet_factory)
